"""Build Helper Functions
"""

import os


VERSION = (1, 0)
__version__ = '.'.join(str(i) for i in VERSION)


def reachable_nodes_in_order(start_node, graph):
    """Returns a new list with all reachable nodes of the graph starting
    from start_node.

    The returned list contains nodes in order of visit and
    doesn't contain any duplicate.

    Arguments:
      start_node = node where the search for reachable nodes begins.
      graph = dictionary representation of a directed acyclic graph.
              Sink nodes don't need to have keys in the dictionary.
    """
    reachable_nodes = []
    stack = graph.get(start_node, [])[::-1]
    visited = set([start_node])
    while stack:
        node = stack[-1]
        if node in visited:
            stack.pop()
            if node not in reachable_nodes:
                reachable_nodes.append(node)
            continue
        visited.add(node)
        sublist = graph.get(node, [])[::-1]
        if sublist:
            stack.extend(sublist)
        else:
            reachable_nodes.append(stack.pop())
    return reachable_nodes


def check_files_exist(file_list):
    """Returns True if all files in `file_list` exist, else raise an IOError.

    Arguments:
      file_list = list of strings with filenames.
    """
    for filename in file_list:
        if not os.path.isfile(filename):
            raise IOError('file not found: {0}'.format(filename))
    return True


def concatenate_files(file_list, output_path):
    """Concatenate files in `file_list` and write them in a new file.

    Arguments:
      file_list = list with filenames
      output_path = path to new file
    """
    with open(output_path, 'wb') as output_file:
        for filename in file_list:
            with open(filename, 'rb') as file:
                output_file.write(file.read())


def build_file(filename, dependency_graph, directory):
    """Create a new file named `filename` using the dependency graph.

    In case of failure, raise an IOError.

    Arguments:
      filename = filename of the output file
      dependency_graph = dictionary where keys are filenames and
                         values are lists with the dependencies files.
      directory = path to the directory where the new file will be created.
    """
    dependencies = reachable_nodes_in_order(filename, dependency_graph)
    try:
        check_files_exist(dependencies)
        concatenate_files(dependencies, os.path.join(directory, filename))
    except IOError as exception:
        raise IOError(
            'cannot build file: {0}\n  {1}\n'.format(filename, exception))
